
package mechanisms;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;


import sim.engine.SimState;
import sim.engine.Steppable;
import simulations.AuctionSimulation;
import simulations.WineSimulation;
import traders.BaseTrader;

/**
 * @author
 *
 */
public class Auction extends TradingMechanism implements Serializable, Steppable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5361625107370943422L;
	
	//buyers sorted from the highest price to the lowest price
	public LinkedList<BaseTrader> buyers;
	//buyers sorted from the lowest price to the highest price
	public LinkedList<BaseTrader> sellers;
	
	public AuctionSimulation simulation;
	
	public int auctionId;

	/**
	 * 
	 */
	public Auction() {
		buyers = new LinkedList<BaseTrader>();
		sellers = new LinkedList<BaseTrader>();
	}
	
	
	public Auction(AuctionSimulation simulation) {
		buyers = new LinkedList<BaseTrader>();
		sellers = new LinkedList<BaseTrader>();
		this.simulation = simulation;
	}
	
	public void addBuyer(BaseTrader buyer){
		buyers.add(buyer);
	}
	
	// English auction mechanism
	@SuppressWarnings("unused")
	public void executeTransaction(){
		BaseTrader seller = sellers.poll();
		BaseTrader buyer = buyers.peekLast();
		double startingPrice;
		if(WineSimulation.AUCTION_MECHANISM_VARIANT==1 || WineSimulation.AUCTION_MECHANISM_VARIANT==2) {
			startingPrice = seller.sellPrice/2.0;
		} else {
			startingPrice = seller.sellPrice;
		}
		if(buyer.buyPrice >= startingPrice){
			double price;
			if(WineSimulation.AUCTION_MECHANISM_VARIANT==0) {
				if(buyers.size()>1){
					price = Math.max(startingPrice, buyers.get(buyers.size()-2).buyPrice);	
				} else {
					price = startingPrice;
				}
			} else if (WineSimulation.AUCTION_MECHANISM_VARIANT==1) {
				if(buyers.size()>1){
					price = Math.max(startingPrice, buyers.get(buyers.size()-2).buyPrice);	
				} else {
					price = startingPrice; //starting price only half of the value
				}
			} else if (WineSimulation.AUCTION_MECHANISM_VARIANT==2) {
				if(buyers.size()>2){
					price = Math.max(startingPrice, buyers.get(buyers.size()-3).buyPrice);// waiting	
				} else if (buyers.size()>1) {
					price = Math.max(startingPrice, buyers.get(buyers.size()-2).buyPrice);
				} else {
					price = startingPrice; //starting price only half of the value
				}
			} else {
				if(buyers.size()>2){
					price = Math.max(startingPrice, buyers.get(buyers.size()-3).buyPrice);// waiting	
				} else if (buyers.size()>1) {
					price = Math.max(startingPrice, buyers.get(buyers.size()-2).buyPrice);
				} else {
					price = startingPrice; //starting price only half of the value
				}
			}
			seller.sell(price);
			buyer.buy(price);
			buyers.pollLast();
			this.simulation.addTransaction(auctionId, buyer.id, seller.id, price);
		} 
	}
	
	public void executeAllTransactions(){
		//order buyers from the lowest
		Collections.sort(buyers, new Comparator<BaseTrader>(){
		     public int compare(BaseTrader bt1, BaseTrader bt2){
		         if(bt1.buyPrice == bt2.buyPrice) {
		        	 return 0;
		         } else {
		        	 return bt1.buyPrice < bt2.buyPrice ? -1 : 1; 
		         }
		     }
		});
		Collections.shuffle(sellers);
		while (!buyers.isEmpty() & !sellers.isEmpty()){
			executeTransaction();
		}
	}
	
	public void addSeller(BaseTrader seller){
		sellers.add(seller);
	}

	
	/* (non-Javadoc)
	 * @see sim.engine.Steppable#step(sim.engine.SimState)
	 */
	@Override
	public void step(SimState simState) {
		@SuppressWarnings("unused")
		AuctionSimulation simulation = (AuctionSimulation) simState;
		executeAllTransactions();
		buyers.clear();
		sellers.clear();
	}

	@Override
	public boolean isTrading(BaseTrader trader) {
		// TODO Auto-generated method stub
		return false;
	}


	@Override
	public void withdrawOrder(BaseTrader trader) {
		// TODO Auto-generated method stub
		
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}

}
