/**
 * 
 */
package mechanisms;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.TreeMap;

import sim.engine.SimState;
import sim.engine.Steppable;
import simulations.LivexSimulation;
import traders.BaseTrader;

/**
 * @author
 *
 */
public class Livex extends TradingMechanism implements Serializable, Steppable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5361625107370943422L;
	
	//buyers sorted from the lowest price to the lowest price
	public TreeMap<Double,LinkedList<BaseTrader>> buyers;
	//sellers sorted from the lowest price to the highest price
	public TreeMap<Double,LinkedList<BaseTrader>> sellers;
	
	//simulation
	public LivexSimulation simulation;

	/**
	 * 
	 */
	public Livex() {
		buyers = new TreeMap<Double,LinkedList<BaseTrader>>();
		sellers = new TreeMap<Double, LinkedList<BaseTrader>>();
	}
	
	/**
	 * 
	 */
	public Livex(LivexSimulation simulation) {
		buyers = new TreeMap<Double,LinkedList<BaseTrader>>();
		sellers = new TreeMap<Double, LinkedList<BaseTrader>>();
		this.simulation = simulation;
	}
	
	public void addBuyer(BaseTrader buyer){
		// no sellers
		if(sellers.isEmpty()){
			if(buyers.containsKey(new Double(buyer.buyPrice))){
				if(!buyers.get(new Double(buyer.buyPrice)).contains(buyer)){ // just in case
					buyers.get(new Double(buyer.buyPrice)).addLast(buyer);	
				}
			} else {
				buyers.put(new Double(buyer.buyPrice), new LinkedList<BaseTrader>());
				buyers.get(new Double(buyer.buyPrice)).addLast(buyer);
			}
		} else {
			if(sellers.firstKey().doubleValue() <= buyer.buyPrice){
				double price = sellers.firstKey().doubleValue();
				BaseTrader seller = sellers.get(sellers.firstKey()).pollFirst(); 
				seller.sell(price);
				if(sellers.get(sellers.firstKey()).isEmpty()){
					sellers.remove(sellers.firstKey());
				}
				buyer.buy(price);
				this.simulation.addTransaction(buyer.id, seller.id, price);
			} else {
				if(buyers.containsKey(new Double(buyer.buyPrice))){
					if(!buyers.get(new Double(buyer.buyPrice)).contains(buyer)){ // just in case
						buyers.get(new Double(buyer.buyPrice)).addLast(buyer);
					}
				} else {
					buyers.put(new Double(buyer.buyPrice), new LinkedList<BaseTrader>());
					buyers.get(new Double(buyer.buyPrice)).addLast(buyer);
				}
			}
		}
	}
	
	public void addSeller(BaseTrader seller){
		//no buyers
		if(buyers.isEmpty()){
			if(sellers.containsKey(new Double(seller.sellPrice))){
				if(!sellers.get(new Double(seller.sellPrice)).contains(seller)){ //just in case
					sellers.get(new Double(seller.sellPrice)).addLast(seller);	
				}
			} else {
				sellers.put(new Double(seller.sellPrice), new LinkedList<BaseTrader>());
				sellers.get(new Double(seller.sellPrice)).addLast(seller);	
			}
		} else {
			if(buyers.lastKey().doubleValue() >= seller.sellPrice){
				double price = buyers.lastKey().doubleValue();
				BaseTrader buyer = buyers.get(buyers.lastKey()).pollFirst();
				buyer.buy(price);
				if(buyers.get(buyers.lastKey()).isEmpty()){
					buyers.remove(buyers.lastKey());
				}
				seller.sell(price);
				this.simulation.addTransaction(buyer.id, seller.id, price);
			} else {
				if(sellers.containsKey(new Double(seller.sellPrice))){
					if(!sellers.get(new Double(seller.sellPrice)).contains(seller)){ //just in case
						sellers.get(new Double(seller.sellPrice)).addLast(seller);
					}
				} else {
					sellers.put(new Double(seller.sellPrice), new LinkedList<BaseTrader>());
					sellers.get(new Double(seller.sellPrice)).addLast(seller);
				}
			}
		}
	}

	
	@Override
	public boolean isTrading(BaseTrader trader) {
		if(trader.hasWine){ //is seller
			if(sellers.containsKey(new Double(trader.sellPrice)) && sellers.get(new Double(trader.sellPrice)).contains(trader)){
				return true;
			} else {
				return false;		
			}
		} else { //is buyer
			if(buyers.containsKey(new Double(trader.buyPrice)) && buyers.get(new Double(trader.buyPrice)).contains(trader)){
				return true;
			} else {
				return false;		
			}
		}
	}

	@Override
	public void withdrawOrder(BaseTrader trader) {
		if(trader.hasWine){ //is seller
			sellers.get(trader.sellPrice).remove(trader);
			if(sellers.get(trader.sellPrice).isEmpty()){
				sellers.remove(trader.sellPrice);
			}
		} else { //is buyer
			buyers.get(trader.buyPrice).remove(trader);
			if(buyers.get(trader.buyPrice).isEmpty()){
				buyers.remove(trader.buyPrice);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see sim.engine.Steppable#step(sim.engine.SimState)
	 */
	@Override
	public void step(SimState simState) {
		//currently empty only for debugging purposes
		@SuppressWarnings("unused")
		LivexSimulation simulation = (LivexSimulation) simState;
		//System.out.println("Livex at step: " + simulation.schedule.getSteps());

	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}



}
