/**
 * 
 */
package mechanisms;

import java.io.Serializable;
import java.util.Collections;
import java.util.Comparator;
import java.util.LinkedList;

import sim.engine.SimState;
import sim.engine.Steppable;
import simulations.OffexSimulation;
import traders.BaseTrader;

/**
 * @author
 * now with classical demand and supply curves 
 *
 */
public class Offex extends TradingMechanism implements Serializable, Steppable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5361625107370943422L;
	
	public LinkedList<BaseTrader> buyers;
	public LinkedList<BaseTrader> sellers;
	
	public OffexSimulation simulation;
	//average price from the selling prices of sellers
	public double averagePrice;
	//id for shop
	public int shopID;
	

	/**
	 * 
	 */
	public Offex() {
		buyers = new LinkedList<BaseTrader>();
		sellers = new LinkedList<BaseTrader>();
	}
	
	public Offex(OffexSimulation simulation) {
		buyers = new LinkedList<BaseTrader>();
		sellers = new LinkedList<BaseTrader>();
		this.simulation = simulation;
	}
	
	public Offex(OffexSimulation simulation, int shopID) {
		buyers = new LinkedList<BaseTrader>();
		sellers = new LinkedList<BaseTrader>();
		this.simulation = simulation;
		this.shopID = shopID;
	}
	
	public void addBuyer(BaseTrader buyer){
		if(!sellers.isEmpty() && buyer.buyPrice>=averagePrice){
			buyer.buy(averagePrice);
			BaseTrader seller = sellers.poll();
			seller.sell(averagePrice);
			buyer.isStillActive = false;
			buyers.remove(buyer);
			this.simulation.addTransaction(shopID, buyer.id, seller.id, averagePrice);
		}
	}
	
	public void addSeller(BaseTrader seller){
	
	}
	
	/**
	 * calculate the average price based on demand and supply 
	 * 
	 */
	
	public void calculateAveragePrice(){
		
		//order buyers from the highest
		Collections.sort(buyers, new Comparator<BaseTrader>(){
		     public int compare(BaseTrader bt1, BaseTrader bt2){
		         if(bt1.buyPrice == bt2.buyPrice) {
		        	 return 0;
		         } else {
		        	 return bt1.buyPrice > bt2.buyPrice ? -1 : 1; 
		         }
		     }
		});
		
				
		//order sellers from the lowest
		Collections.sort(sellers, new Comparator<BaseTrader>(){
		     public int compare(BaseTrader bt1, BaseTrader bt2){
		         if(bt1.sellPrice == bt2.sellPrice) {
		        	 return 0;
		         } else {
		        	 return bt1.sellPrice < bt2.sellPrice ? -1 : 1; 
		         }
		     }
		});
		
		//calculate the price at which demand and supply curve intersect
		
		if(!sellers.isEmpty() & !buyers.isEmpty()){
			int intersect = Math.min(sellers.size(), buyers.size());
			do {
				intersect--;
			} while (sellers.get(intersect).sellPrice>buyers.get(intersect).buyPrice & intersect >0);
			averagePrice = Math.max((double) sellers.get(intersect).sellPrice, 
					((double) sellers.get(intersect).sellPrice + (double) buyers.get(intersect).buyPrice)/2.0);	
		}
		
		Collections.shuffle(sellers);
	}

	
	/* (non-Javadoc)
	 * @see sim.engine.Steppable#step(sim.engine.SimState)
	 */
	@Override
	public void step(SimState simState) {
		@SuppressWarnings("unused")
		OffexSimulation simulation = (OffexSimulation) simState;
		calculateAveragePrice();
	}

	
	@Override
	public boolean isTrading(BaseTrader trader) {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void withdrawOrder(BaseTrader trader) {
		// TODO Auto-generated method stub	
	}
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub

	}
}
