/**
 * 
 */
package simulations;

import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import com.opencsv.CSVReader;

import ec.util.MersenneTwisterFast;
import mechanisms.Auction;
import traders.SimpleTrader;

/**
 * @author
 * livex simulation with base traders
 *
 */
public class AuctionSimulation extends WineSimulation  {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = -560524368714131744L;
	
	public int numberShops;
	public int[] numberShopsRange = new int[]{5,15};
	public int[] numberShopsArray = new int[NUMBER_SOBOL];
	
	public ArrayList<Auction> tradingMechanisms;


	public AuctionSimulation(long seed, int threadID) {
		super(seed);
		this.threadID = threadID;
		simulationName = "AUCTION";
		tradingMechanisms = new ArrayList<Auction>(); 
	}
	
	/**
	 * @param random
	 */
	public AuctionSimulation(MersenneTwisterFast random) {
		super(random);
		// TODO Auto-generated constructor stub
	}
	
	
	
	public void readSobol(){
		int i =0;
		String csvFile = "sobol.csv";
		try {
            reader = new CSVReader(new FileReader(CSV_FOLDER + csvFile), ';');
            reader.readNext(); //because of header
            String[] line;
            while ((line = reader.readNext()) != null) {
                readSobolSingleLine(i,line);
                i++;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
	}

	public void readSobolSingleLine(int i, String[] line) {
		double weigth;
		weigth = Double.parseDouble(line[1].replace(",", "."));
		numberBuyersArray[i] = getSobolNumber(weigth, numberBuyersRange);
		weigth = Double.parseDouble(line[2].replace(",", "."));
		numberSellersArray[i] = getSobolNumber(weigth, numberSellersRange);
		weigth = Double.parseDouble(line[3].replace(",", "."));
		muBuyersArray[i] = getSobolNumber(weigth, muBuyersRange);
		weigth = Double.parseDouble(line[4].replace(",", "."));
		sigmaBuyersArray[i] = getSobolNumber(weigth, sigmaBuyersRange);
		weigth = Double.parseDouble(line[5].replace(",", "."));
		muSellersArray[i] = getSobolNumber(weigth, muSellersRange);
		weigth = Double.parseDouble(line[6].replace(",", "."));
		sigmaSellersArray[i] = getSobolNumber(weigth, sigmaSellersRange);
		weigth = Double.parseDouble(line[7].replace(",", "."));
		lambdaInactivationArray[i] = getSobolNumber(weigth, lambdaInactivationRange);
		weigth = Double.parseDouble(line[8].replace(",", "."));
		lambdaPlaceOrderArray[i] = getSobolNumber(weigth, lambdaPlaceOrderRange);
		weigth = Double.parseDouble(line[9].replace(",", "."));
		lambdaRemoveOrderArray[i] = getSobolNumber(weigth, lambdaRemoveOrderRange);
		weigth = Double.parseDouble(line[10].replace(",", "."));
		provisionArray[i] = getSobolNumber(weigth, provisionRange);
		weigth = Double.parseDouble(line[11].replace(",", "."));
		numberShopsArray[i] = getSobolNumber(weigth, numberShopsRange);
	}

	public void addTransaction(int auctionId, int buyerId, int sellerId, double price){
		transactions.add(new double[]{this.parameterID, loopID, this.schedule.getSteps(), auctionId, buyerId, sellerId, price});
	}
	//setups
	
	public void setUp(){
		buyers.clear();
		sellers.clear();
		tradingMechanisms.clear();
		setUpBuyers();
		setUpSellers();
		setUpTradingMechanism();
	}
	
	
	//setup buyers & sellers
	
	public void setUpBuyers(){
		for (int i = 0; i < numberBuyers; i++) {
			SimpleTrader trader = new SimpleTrader(currentId++, true,
					Math.max((1+muBuyers + randomBuyers.nextGaussian() * sigmaBuyers)*AuctionSimulation.START_VALUE,AuctionSimulation.MIN_VALUE),
							this.provision);
			buyers.add(trader);
		}
	}
	
	public void setUpSellers(){
		for (int i = 0; i < numberSellers; i++) {
			SimpleTrader trader = new SimpleTrader(currentId++,false,
					Math.max((1-muSellers + randomSellers.nextGaussian() * sigmaSellers)*AuctionSimulation.START_VALUE,AuctionSimulation.MIN_VALUE),
							this.provision);
			sellers.add(trader);
		}
	}

	//setUp trading mechanisms
	public void setUpTradingMechanism(){
		for (int i = 0; i < numberShops; i++) {
			this.tradingMechanisms.add(new Auction(this));
		}
		//randomly allocate traders to auction trading mechanisms
		for(SimpleTrader trader : buyers){
			int shopID = randomPlace.nextInt(numberShops);
			trader.tradingMechanism = tradingMechanisms.get(shopID);
		}
		for(SimpleTrader trader : sellers){
			int shopID = randomShopSelection.nextInt(numberShops);
			trader.tradingMechanism = tradingMechanisms.get(shopID);
		}
	}
	
	@Override
	public void setUpAll() {
		setUp();
	}

	@Override
	public void scheduleAll() {
		for (SimpleTrader buyer: buyers){	
			schedule.scheduleRepeating(buyer, 0, 1.0);
		}
		for (SimpleTrader seller: sellers){	
			schedule.scheduleRepeating(seller, 0, 1.0);
		}
		for (Auction tradingMechanism: tradingMechanisms) {
			schedule.scheduleRepeating(tradingMechanism, 1, 1.0);
		}
	}

	@Override
	public void updateParameters(int i) {
		numberBuyers=numberBuyersArray[i];
		numberSellers=numberSellersArray[i];
		muBuyers=muBuyersArray[i];
		sigmaBuyers=sigmaBuyersArray[i];
		muSellers=muSellersArray[i];
		sigmaSellers=sigmaSellersArray[i];
		lambdaInactivation=lambdaInactivationArray[i];
		lambdaPlaceOrder=lambdaPlaceOrderArray[i];
		lambdaRemoveOrder=lambdaRemoveOrderArray[i];
		provision=provisionArray[i];
		numberShops=numberShopsArray[i];
		
		parameters.add(new double[]{i,numberBuyers,numberSellers,muBuyers,sigmaBuyers, muSellers, sigmaSellers, lambdaInactivation,
		lambdaPlaceOrder,lambdaRemoveOrder,provision, numberShops});
	}	

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {
		super.run();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		//number of simulation adjusted to the number of available cores
		AuctionSimulation aSimulator0 = new AuctionSimulation(1234, 0);
		aSimulator0.startThread();
		AuctionSimulation aSimulator1 = new AuctionSimulation(1234, 1);
		aSimulator1.startThread();
	}

}
