/**
 * 
 */
package simulations;

import ec.util.MersenneTwisterFast;
import mechanisms.Livex;
import traders.SimpleTrader;

/**
 * @author
 * livex simulation with base traders
 *
 */
public class LivexSimulation extends WineSimulation  {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = -560524368714131744L;


	public LivexSimulation(long seed, int threadID) {
		super(seed);
		this.threadID = threadID;
		simulationName = "LIVEX";
	}
	
	/**
	 * @param random
	 */
	public LivexSimulation(MersenneTwisterFast random) {
		super(random);
		// TODO Auto-generated constructor stub
	}
	
	
	
	public void readSobolSingleLine(int i, String[] line) {
		double weigth;
		weigth = Double.parseDouble(line[1].replace(",", "."));
		numberBuyersArray[i] = getSobolNumber(weigth, numberBuyersRange);
		weigth = Double.parseDouble(line[2].replace(",", "."));
		numberSellersArray[i] = getSobolNumber(weigth, numberSellersRange);
		weigth = Double.parseDouble(line[3].replace(",", "."));
		muBuyersArray[i] = getSobolNumber(weigth, muBuyersRange);
		weigth = Double.parseDouble(line[4].replace(",", "."));
		sigmaBuyersArray[i] = getSobolNumber(weigth, sigmaBuyersRange);
		weigth = Double.parseDouble(line[5].replace(",", "."));
		muSellersArray[i] = getSobolNumber(weigth, muSellersRange);
		weigth = Double.parseDouble(line[6].replace(",", "."));
		sigmaSellersArray[i] = getSobolNumber(weigth, sigmaSellersRange);
		weigth = Double.parseDouble(line[7].replace(",", "."));
		lambdaInactivationArray[i] = getSobolNumber(weigth, lambdaInactivationRange);
		weigth = Double.parseDouble(line[8].replace(",", "."));
		lambdaPlaceOrderArray[i] = getSobolNumber(weigth, lambdaPlaceOrderRange);
		weigth = Double.parseDouble(line[9].replace(",", "."));
		lambdaRemoveOrderArray[i] = getSobolNumber(weigth, lambdaRemoveOrderRange);
		weigth = Double.parseDouble(line[10].replace(",", "."));
		provisionArray[i] = getSobolNumber(weigth, provisionRange);
	}

	
	
	//setups
	
	public void setUp(){
		buyers.clear();
		sellers.clear();
		setUpBuyers();
		setUpSellers();
		setUpTradingMechanism();
	}
	
	
	//setup buyers & sellers
	
	public void setUpBuyers(){
		for (int i = 0; i < numberBuyers; i++) {
			SimpleTrader trader = new SimpleTrader(currentId++, true,
					Math.max((1+muBuyers + randomBuyers.nextGaussian() * sigmaBuyers)*LivexSimulation.START_VALUE,LivexSimulation.MIN_VALUE),
							this.provision);
			buyers.add(trader);
		}
	}
	
	public void setUpSellers(){
		for (int i = 0; i < numberSellers; i++) {
			SimpleTrader trader = new SimpleTrader(currentId++,false,
					Math.max((1-muSellers + randomSellers.nextGaussian() * sigmaSellers)*LivexSimulation.START_VALUE,LivexSimulation.MIN_VALUE),
							this.provision);
			sellers.add(trader);
		}
	}

	//setUp trading mechanisms
	public void setUpTradingMechanism(){
		this.tradingMechanism = new Livex(this);
		for(SimpleTrader trader : buyers){
			trader.tradingMechanism = tradingMechanism;
		}
		for(SimpleTrader trader : sellers){
			trader.tradingMechanism = tradingMechanism;
		}
	}
	
	@Override
	public void setUpAll() {
		setUp();
	}

	@Override
	public void scheduleAll() {
		for (SimpleTrader buyer: buyers){	
			schedule.scheduleRepeating(buyer, 0, 1.0);
		}
		for (SimpleTrader seller: sellers){	
			schedule.scheduleRepeating(seller, 0, 1.0);
		}
		schedule.scheduleRepeating(this.tradingMechanism, 1, 1.0);
	}

	@Override
	public void updateParameters(int i) {
		numberBuyers=numberBuyersArray[i];
		numberSellers=numberSellersArray[i];
		muBuyers=muBuyersArray[i];
		sigmaBuyers=sigmaBuyersArray[i];
		muSellers=muSellersArray[i];
		sigmaSellers=sigmaSellersArray[i];
		lambdaInactivation=lambdaInactivationArray[i];
		lambdaPlaceOrder=lambdaPlaceOrderArray[i];
		lambdaRemoveOrder=lambdaRemoveOrderArray[i];
		provision=provisionArray[i];
		
		parameters.add(new double[]{i,numberBuyers,numberSellers,muBuyers,sigmaBuyers, muSellers, sigmaSellers, lambdaInactivation,
		lambdaPlaceOrder,lambdaRemoveOrder,provision});
	}	

	/* (non-Javadoc)
	 * @see java.lang.Runnable#run()
	 */
	@Override
	public void run() {
		super.run();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		//number of simulation adjusted to the number of available cores
		LivexSimulation aSimulator0 = new LivexSimulation(1234, 0);
		aSimulator0.startThread();
		LivexSimulation aSimulator1 = new LivexSimulation(1234, 1);
		aSimulator1.startThread();
	}

}
